/*============================================================================
  CMake - Cross Platform Makefile Generator
  Copyright 2004-2009 Kitware, Inc.
  Copyright 2004 Alexander Neundorf (neundorf@kde.org)

  Distributed under the OSI-approved BSD License (the "License");
  see accompanying file Copyright.txt for details.

  This software is distributed WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the License for more information.
============================================================================*/
#include "cmExtraCodeLiteGenerator.h"
#include "cmGlobalUnixMakefileGenerator3.h"
#include "cmLocalUnixMakefileGenerator3.h"
#include "cmMakefile.h"
#include "cmake.h"
#include "cmSourceFile.h"
#include "cmGeneratedFileStream.h"
#include "cmSystemTools.h"

#include <cmsys/SystemTools.hxx>
#include <cmsys/Directory.hxx>

//----------------------------------------------------------------------------
void cmExtraCodeLiteGenerator::GetDocumentation(cmDocumentationEntry& entry, const char*) const
{
  entry.Name = this->GetName();
  entry.Brief = "Generates CodeLite project files.";
  entry.Full =
    "Project files for CodeLite will be created in the top directory ...";
}

cmExtraCodeLiteGenerator::cmExtraCodeLiteGenerator()
:cmExternalMakefileProjectGenerator()
{
  this->SupportedGlobalGenerators.push_back("Unix Makefiles");
}


void cmExtraCodeLiteGenerator::Generate()
{
  
 // Hold root tree information for creating the workspace 
 std::string workspaceProjectName;
 std::string workspaceOutputDir;
 std::string workspaceFileName;
 std::string workspaceSourcePath;
 // list of debug projects 
 std::string lprjdebug;
 // list of release projects
 std::string lprjrelease;
 
 cmGeneratedFileStream fout; 

// loop projects and locate the root project.
// and extract the information for creating the worspace 

for (std::map<cmStdString, std::vector<cmLocalGenerator*> >::const_iterator
       it = this->GlobalGenerator->GetProjectMap().begin();
      it!= this->GlobalGenerator->GetProjectMap().end();
      ++it)
    {
        const cmMakefile* mf =it->second[0]->GetMakefile();
        // 
        if (strcmp(mf->GetStartOutputDirectory(), mf->GetHomeOutputDirectory()) == 0)
        {
                workspaceOutputDir = mf->GetStartOutputDirectory();
                workspaceProjectName = mf->GetProjectName();
                workspaceSourcePath = mf->GetHomeDirectory();
                workspaceFileName = workspaceOutputDir+"/";
                workspaceFileName+= workspaceProjectName + ".workspace";

                fout.Open(workspaceFileName.c_str(),false,false);
                fout <<    "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
                            "<CodeLite_Workspace Name=\"" << workspaceProjectName << "\" Database=\"./" << workspaceProjectName << ".tags\">\n";
        }
    }

  // for each sub project in the project create a codeblocks project
  for (std::map<cmStdString, std::vector<cmLocalGenerator*> >::const_iterator
       it = this->GlobalGenerator->GetProjectMap().begin();
      it!= this->GlobalGenerator->GetProjectMap().end();
      ++it)
    {
            // retrive project information
            const cmMakefile* mf =it->second[0]->GetMakefile();
            std::string outputDir =mf->GetStartOutputDirectory();
            std::string projectName =mf->GetProjectName();
            std::string filename=outputDir+"/" + projectName + ".project";

    // create a project file
    this->CreateProjectFile(it->second);
    fout << "  <Project Name=\"" << projectName << "\" Path=\"" << filename << "\" Active=\"Yes\"/>\n";
    lprjdebug += "<Project Name=\"" + projectName + "\" ConfigName=\"Debug\"/>\n";
    lprjrelease += "<Project Name=\"" + projectName + "\" ConfigName=\"Release\"/>\n";
    }
    
    fout <<                                      "  <BuildMatrix>\n"
                                     "    <WorkspaceConfiguration Name=\"Debug\" Selected=\"yes\">\n"
                                     "      " << lprjdebug << ""
                                     "    </WorkspaceConfiguration>\n"
                                     "    <WorkspaceConfiguration Name=\"Release\" Selected=\"yes\">\n"
                                     "      " << lprjrelease << ""
                                     "    </WorkspaceConfiguration>\n"
                                     "  </BuildMatrix>\n"
                                     "</CodeLite_Workspace>\n";
}


/* create the project file */
void cmExtraCodeLiteGenerator::CreateProjectFile(
                                     const std::vector<cmLocalGenerator*>& lgs)
{
  const cmMakefile* mf=lgs[0]->GetMakefile();
  std::string outputDir=mf->GetStartOutputDirectory();
  std::string projectName=mf->GetProjectName();

  std::string filename=outputDir+"/";
  filename+=projectName+".project";

  this->CreateNewProjectFile(lgs, filename);
}


void cmExtraCodeLiteGenerator
  ::CreateNewProjectFile(const std::vector<cmLocalGenerator*>& lgs,
                         const std::string& filename)
{
  const cmMakefile* mf=lgs[0]->GetMakefile();
  cmGeneratedFileStream fout(filename.c_str());
  if(!fout)
    {
    return;
    }

  // figure out the compiler
  //std::string compiler = this->GetCBCompilerId(mf);
  std::string make = mf->GetRequiredDefinition("CMAKE_MAKE_PROGRAM");
  std::string workspaceSourcePath = mf->GetHomeDirectory();
   std::string workspaceOutputDir =  mf->GetHomeOutputDirectory();

////////////////////////////////////
fout << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
                                    "<CodeLite_Project Name=\"" << mf->GetProjectName() << "\" InternalType=\"\">\n"
                                    "  <Plugins>\n"
                                    "    <Plugin Name=\"qmake\">\n"
                                    "      <![CDATA[00010001N0005Debug000000000000]]>\n"
                                    "    </Plugin>\n"
                                    "  </Plugins>\n"
                                    "\n";



  // Collect all used source files in the project
  // Sort them into two containers, one for C/C++ implementation files
  // which may have an acompanying header, one for all other files
  std::map<std::string, cmSourceFile*> cFiles;
  std::set<std::string> otherFiles;
  for (std::vector<cmLocalGenerator*>::const_iterator lg=lgs.begin();
       lg!=lgs.end(); lg++)
    {
    cmMakefile* makefile=(*lg)->GetMakefile();
    cmTargets& targets=makefile->GetTargets();
    for (cmTargets::iterator ti = targets.begin();
         ti != targets.end(); ti++)
      {
      switch(ti->second.GetType())
        {
        case cmTarget::EXECUTABLE:
        case cmTarget::STATIC_LIBRARY:
        case cmTarget::SHARED_LIBRARY:
        case cmTarget::MODULE_LIBRARY:
          {
          const std::vector<cmSourceFile*>&sources=ti->second.GetSourceFiles();
          for (std::vector<cmSourceFile*>::const_iterator si=sources.begin();
               si!=sources.end(); si++)
            {
            // check whether it is a C/C++ implementation file
            bool isCFile = false;
            if ((*si)->GetLanguage() && (*(*si)->GetLanguage() == 'C'))
              {
              for(std::vector<std::string>::const_iterator
                  ext = mf->GetSourceExtensions().begin();
                  ext !=  mf->GetSourceExtensions().end(); 
                  ++ext)
                {
                if ((*si)->GetExtension() == *ext)
                  {
                  isCFile = true;
                  break;
                  }
                }
              }

            // then put it accordingly into one of the two containers
            if (isCFile)
              {
              cFiles[(*si)->GetFullPath()] = *si ;
              }
            else
              {
              otherFiles.insert((*si)->GetFullPath());
              }
            }
          }
        default:  // intended fallthrough
          break;
        }
      }
    }

  // The following loop tries to add header files matching to implementation
  // files to the project. It does that by iterating over all source files,
  // replacing the file name extension with ".h" and checks whether such a 
  // file exists. If it does, it is inserted into the map of files.
  // A very similar version of that code exists also in the kdevelop
  // project generator.
  for (std::map<std::string, cmSourceFile*>::const_iterator 
       sit=cFiles.begin();
       sit!=cFiles.end();
       ++sit)
    {
    std::string headerBasename=cmSystemTools::GetFilenamePath(sit->first);
    headerBasename+="/";
    headerBasename+=cmSystemTools::GetFilenameWithoutExtension(sit->first);

    // check if there's a matching header around
    for(std::vector<std::string>::const_iterator
        ext = mf->GetHeaderExtensions().begin();
        ext !=  mf->GetHeaderExtensions().end(); 
        ++ext)
      {
      std::string hname=headerBasename;
      hname += ".";
      hname += *ext;
      // if it's already in the set, don't check if it exists on disk
      std::set<std::string>::const_iterator headerIt=otherFiles.find(hname);
      if (headerIt != otherFiles.end())
        {
        break;
        }

      if(cmSystemTools::FileExists(hname.c_str()))
        {
        otherFiles.insert(hname);
        break;
        }
      }
    }
//const char* gVirtualFileTemplate = "<File Name=\"RPLC_FILE_PATH\"/>\n"
//const char* gVirtualDirStartTemplate = "<VirtualDirectory Name=\"RPLC_DIR_NAME\">\n"
//const char* gVirtualDirEndTemplate = "<VirtualDirectory/>\n"
  fout<<        "<VirtualDirectory Name=\"" << mf->GetProjectName() << "\">\n";

  // insert all source files in the CodeBlocks project
  // first the C/C++ implementation files, then all others
  for (std::map<std::string, cmSourceFile*>::const_iterator 
       sit=cFiles.begin();
       sit!=cFiles.end();
       ++sit)
    {
        fout<<        "<File Name=\"" << sit->first << "\"/>\n";
    }
  for (std::set<std::string>::const_iterator 
       sit=otherFiles.begin();
       sit!=otherFiles.end();
       ++sit)
  {
        fout<<        "<File Name=\"" << sit->c_str() << "\"/>\n";
  }
  fout<<        "</VirtualDirectory>\n";


  fout<<                              "\n"
                                    "  <Settings Type=\"Dynamic Library\">\n"
                                    "    <Configuration Name=\"Debug\" CompilerType=\"gnu g++\" DebuggerType=\"GNU gdb debugger\" Type=\"Dynamic Library\" BuildCmpWithGlobalSettings=\"append\" BuildLnkWithGlobalSettings=\"append\" BuildResWithGlobalSettings=\"append\">\n"
                                    "      <Compiler Options=\"-g\" Required=\"yes\" PreCompiledHeader=\"\">\n"
                                    "        <IncludePath Value=\".\"/>\n"
                                    "      </Compiler>\n"
                                    "      <Linker Options=\"\" Required=\"yes\"/>\n"
                                    "      <ResourceCompiler Options=\"\" Required=\"no\"/>\n"
                                    "      <General OutputFile=\"\" IntermediateDirectory=\"./Debug\" Command=\"\" CommandArguments=\"\" WorkingDirectory=\"$(IntermediateDirectory)\" PauseExecWhenProcTerminates=\"yes\"/>\n"
                                    "      <Debugger IsRemote=\"no\" RemoteHostName=\"\" RemoteHostPort=\"\" DebuggerPath=\"\">\n"
                                    "        <PostConnectCommands/>\n"
                                    "        <StartupCommands/>\n"
                                    "      </Debugger>\n"
                                    "      <PreBuild/>\n"
                                    "      <PostBuild/>\n"
                                    "      <CustomBuild Enabled=\"yes\">\n"
                                    "        <RebuildCommand/>\n"
                                    "        <CleanCommand>make clean</CleanCommand>\n"
                                    "        <BuildCommand>make -j3</BuildCommand>\n"
                                    "        <PreprocessFileCommand/>\n"
                                    "        <SingleFileCommand>make -j3</SingleFileCommand>\n"
                                    "        <MakefileGenerationCommand>cmake-gui " << workspaceOutputDir << "</MakefileGenerationCommand>\n"
                                    "        <ThirdPartyToolName>Other</ThirdPartyToolName>\n"
                                    "        <WorkingDirectory>$(WorkspacePath)</WorkingDirectory>\n"
                                    "      </CustomBuild>\n"
                                    "      <AdditionalRules>\n"
                                    "        <CustomPostBuild/>\n"
                                    "        <CustomPreBuild/>\n"
                                    "      </AdditionalRules>\n"
                                    "    </Configuration>\n"
                                    "    <Configuration Name=\"Release\" CompilerType=\"gnu g++\" DebuggerType=\"GNU gdb debugger\" Type=\"Dynamic Library\" BuildCmpWithGlobalSettings=\"append\" BuildLnkWithGlobalSettings=\"append\" BuildResWithGlobalSettings=\"append\">\n"
                                    "      <Compiler Options=\"\" Required=\"yes\" PreCompiledHeader=\"\">\n"
                                    "        <IncludePath Value=\".\"/>\n"
                                    "      </Compiler>\n"
                                    "      <Linker Options=\"-O2\" Required=\"yes\"/>\n"
                                    "      <ResourceCompiler Options=\"\" Required=\"no\"/>\n"
                                    "      <General OutputFile=\"\" IntermediateDirectory=\"./Release\" Command=\"\" CommandArguments=\"\" WorkingDirectory=\"$(IntermediateDirectory)\" PauseExecWhenProcTerminates=\"yes\"/>\n"
                                    "      <Debugger IsRemote=\"no\" RemoteHostName=\"\" RemoteHostPort=\"\" DebuggerPath=\"\">\n"
                                    "        <PostConnectCommands/>\n"
                                    "        <StartupCommands/>\n"
                                    "      </Debugger>\n"
                                    "      <PreBuild/>\n"
                                    "      <PostBuild/>\n"
                                    "      <CustomBuild Enabled=\"yes\">\n"
                                    "        <RebuildCommand/>\n"
                                    "        <CleanCommand>make clean</CleanCommand>\n"
                                    "        <BuildCommand>make</BuildCommand>\n"
                                    "        <PreprocessFileCommand/>\n"
                                    "        <SingleFileCommand/>\n"
                                    "        <MakefileGenerationCommand/>\n"
                                    "        <ThirdPartyToolName>None</ThirdPartyToolName>\n"
                                    "        <WorkingDirectory>$(WorkspacePath)</WorkingDirectory>\n"
                                    "      </CustomBuild>\n"
                                    "      <AdditionalRules>\n"
                                    "        <CustomPostBuild/>\n"
                                    "        <CustomPreBuild/>\n"
                                    "      </AdditionalRules>\n"
                                    "    </Configuration>\n"
                                    "    <GlobalSettings>\n"
                                    "      <Compiler Options=\"\">\n"
                                    "        <IncludePath Value=\".\"/>\n"
                                    "      </Compiler>\n"
                                    "      <Linker Options=\"\">\n"
                                    "        <LibraryPath Value=\".\"/>\n"
                                    "      </Linker>\n"
                                    "      <ResourceCompiler Options=\"\"/>\n"
                                    "    </GlobalSettings>\n"
                                    "  </Settings>\n"
                                    "  <Dependencies Name=\"Debug\"/>\n"
                                    "</CodeLite_Project>\n";


}


